# redux-saga-retry

[![NPM Package](https://badge.fury.io/js/redux-saga-retry.svg)](https://www.npmjs.com/package/redux-saga-retry) [![Build Status](https://travis-ci.com/amarofashion/redux-saga-retry.svg?branch=master)](https://travis-ci.com/amarofashion/redux-saga-retry) [![Maintainability](https://api.codeclimate.com/v1/badges/c9742c793415a6dc603c/maintainability)](https://codeclimate.com/github/amarofashion/redux-saga-retry/maintainability) [![Test Coverage](https://api.codeclimate.com/v1/badges/c9742c793415a6dc603c/test_coverage)](https://codeclimate.com/github/amarofashion/redux-saga-retry/test_coverage)

Add a retry machine to your saga by just wrapping it!

```js
// before
takeLatest('GET_COFFEE', getCoffee)

// after
takeLatest('GET_COFFEE', retry(getCoffee))
```

It will retry your saga when a failure action happens.

## Highlights

- 🏖 **Easy to use:** Simply wrap your saga
- 🔼 **Incremental adoption:** Just add to the sagas that need it
- 🛠 **Flexible and granular:** Personalize the options to fit your needs
- 🟦 **Typescript ready:** Nicely typed (but accepting improvements too)


## Install
```sh
$ npm install redux-saga-retry
```


## Usage

Import the module and wrap your saga:
```js
import { retry } from 'redux-saga-retry';

function* getCoffee(action) { ... }

export default function* root() {
  yield all([
    takeLatest('GET_COFFEE', retry(getCoffee)),
  ]);
}
```

If your saga yields an action with the type ending with `_FAILURE` it will start it over:

```js
function* getCoffee(action) {
  try {
    yield call(...);

    yield put({ type: 'GET_COFFEE_SUCCESS' });
  } catch (error) {
    // This will trigger the retry machine
		yield put({ type: 'GET_COFFEE_FAILURE' });
  }
}
```

Please note that the last try will not stop the execution, and your saga will run until its end.


## API

## `retry(saga, [options])`

Wraps a saga function and returns a retryable version of it.

The `options` argument is an object that you can tune how the retry works.


### Options

#### condition: `RegExp | (v: any) => boolean`

The condition to tell if the retry should happen.

_Default: `/_FAILURE$/`_

As a `RegExp`, it listen for redux-saga's action that matches the provided regex.

As a `function`, you are responsible to decide if it should retry the execution.


#### retries: `number`

The maximum number of retries the saga will run. Doesn't include the original run.

_Default: `3`_

It accepts `Infinity` as value.

You can override this value for each run by setting a `meta.retries` on the action that you dispatch to the store:
```js
dispatch({
  type: 'GET_COFFEE',
  meta: { retries: 42 }, // that's a lot of retries
});
```


#### backoff: `(attempt: number) => number`

A function to sparse the retries, where `attempt` is the number of the current attempt (starting on `0`) and returns how long to wait before trying again, in milliseconds.

We provide some basic functions (linearGrowth and exponentialGrowth) but you are free to implement your own.

_Default: `exponentialGrowth`_


## Examples

### Example 1

- matches any action ending on `_FAIL`, `_FAILED` or `_FAILURE`
- retry up to 4 times
- sparse the tries by 400ms, 800ms, 1200ms...

```js
import { retry, linearGrowth } from 'redux-saga-retry';

...

retry(getCoffee, {
  condition: /_FAIL(ED|URE)?$/,
  backoff: linearGrowth,
  retries: 4,
})
```


### Example 2

- custom condition: checks for `PUT` effects in which the action type ends on `_FAILURE` but not if the `payload.status` is `401` (maybe you want to handle the authorization before try again).
- retry up to 3 times (default)
- sparse the tries by an initial delay of 1400ms followed by 400ms, 800ms, 1200ms...

```js
import { retry } from 'redux-saga-retry';

function conditionFn(value) {
  if (value?.type !== 'PUT') {
    return false;  // not interested in other effects
  }

  const { type, payload } = value.payload.action;

  return type.endsWith('_FAILURE') && payload.status !== 401;
}

const backoffFn = i => i === 0 ? 1400 : 400 * i;

...

retry(getCoffee, {
  condition: conditionFn,
  backoff: backoffFn,
})
```

## License

MIT